From: "Yinghai Lu" <yhlu.kernel@gmail.com>
To: "Linus Torvalds" <torvalds@linux-foundation.org>
Cc: "Rafael J. Wysocki" <rjw@sisk.pl>,
"Linux Kernel Mailing List" <linux-kernel@vger.kernel.org>,
"Jeff Garzik" <jeff@garzik.org>, "Tejun Heo" <htejun@gmail.com>,
"Ingo Molnar" <mingo@elte.hu>,
"David Witbrodt" <dawitbro@sbcglobal.net>,
"Andrew Morton" <akpm@linux-foundation.org>,
"Kernel Testers" <kernel-testers@vger.kernel.org>
Subject: Re: Linux 2.6.27-rc5: System boot regression caused by commit a2bd7274b47124d2fc4dfdb8c0591f545ba749dd
Date: Fri, 29 Aug 2008 20:07:18 -0700 [thread overview]
Message-ID: <86802c440808292007t3588edfnef95b723320ff023@mail.gmail.com> (raw)
In-Reply-To: <alpine.LFD.1.10.0808291954410.3300@nehalem.linux-foundation.org>
On Fri, Aug 29, 2008 at 7:56 PM, Linus Torvalds
<torvalds@linux-foundation.org> wrote:
>
>
> On Fri, 29 Aug 2008, Linus Torvalds wrote:
>>
>> Yes. And I do think this is a workable model.
>
> Ok, and here's the patch to do
>
> insert_resource_expand_to_fit(root, new);
>
> and while I still haven't actually tested it, it looks sane and compiles
> to code that also looks sane.
>
> I'll happily commit this as basic infrastructure as soon as somebody ack's
> it and tests that it works (and I'll try it myself soon enough, just for
> fun)
we need to use insert_resource_split_to_fit instead...
otherwise __request_region will not be happy.
have one shrink one
only work with
|----------------|
|---------------------|
still has problem with
|----------------| |------------| |-----------|
|------------------------------------|
need to get rid of middle one too.
YH
---
arch/x86/kernel/e820.c | 20 +++++++++++++-
include/linux/ioport.h | 2 +
kernel/resource.c | 66 ++++++++++++++++++++++++++++++++++++++++---------
3 files changed, 74 insertions(+), 14 deletions(-)
Index: linux-2.6/arch/x86/kernel/e820.c
===================================================================
--- linux-2.6.orig/arch/x86/kernel/e820.c
+++ linux-2.6/arch/x86/kernel/e820.c
@@ -1319,8 +1319,24 @@ void __init e820_reserve_resources_late(
res = e820_res;
for (i = 0; i < e820.nr_map; i++) {
- if (!res->parent && res->end)
- insert_resource(&iomem_resource, res);
+#if 1
+ /* test for shrink_with fit */
+ if (!res->parent && res->end) {
+ if (res->start == 0xe0000000)
+ res->start = 0xde000000;
+ }
+#endif
+
+ if (!res->parent && res->end &&
insert_resource(&iomem_resource, res)) {
+
+ printk(KERN_WARNING "found conflict for %s
[%08llx, %08llx], try to insert with shrink\n",
+ res->name, res->start, res->end);
+
+ insert_resource_shrink_to_fit(&iomem_resource, res);
+
+ printk(KERN_WARNING " shrink to %s [%08llx,
%08llx]\n",
+ res->name, res->start, res->end);
+ }
res++;
}
}
Index: linux-2.6/include/linux/ioport.h
===================================================================
--- linux-2.6.orig/include/linux/ioport.h
+++ linux-2.6/include/linux/ioport.h
@@ -108,6 +108,8 @@ extern struct resource iomem_resource;
extern int request_resource(struct resource *root, struct resource *new);
extern int release_resource(struct resource *new);
+extern void insert_resource_shrink_to_fit(struct resource *root,
+ struct resource *new);
extern int insert_resource(struct resource *parent, struct resource *new);
extern int allocate_resource(struct resource *root, struct resource *new,
resource_size_t size, resource_size_t min,
Index: linux-2.6/kernel/resource.c
===================================================================
--- linux-2.6.orig/kernel/resource.c
+++ linux-2.6/kernel/resource.c
@@ -363,32 +363,30 @@ int allocate_resource(struct resource *r
EXPORT_SYMBOL(allocate_resource);
/**
- * insert_resource - Inserts a resource in the resource tree
+ * __insert_resource - Inserts a resource in the resource tree
* @parent: parent of the new resource
* @new: new resource to insert
*
* Returns 0 on success, -EBUSY if the resource can't be inserted.
*
- * This function is equivalent to request_resource when no conflict
+ * This function is equivalent to __request_resource when no conflict
* happens. If a conflict happens, and the conflicting resources
* entirely fit within the range of the new resource, then the new
* resource is inserted and the conflicting resources become children of
* the new resource.
*/
-int insert_resource(struct resource *parent, struct resource *new)
+static struct resource *__insert_resource(struct resource *parent,
struct resource *new)
{
- int result;
+ struct resource *ret_res;
struct resource *first, *next;
- write_lock(&resource_lock);
-
for (;; parent = first) {
- result = 0;
+ ret_res = NULL;
first = __request_resource(parent, new);
if (!first)
goto out;
- result = -EBUSY;
+ ret_res = first;
if (first == parent)
goto out;
@@ -400,15 +398,17 @@ int insert_resource(struct resource *par
for (next = first; ; next = next->sibling) {
/* Partial overlap? Bad, and unfixable */
- if (next->start < new->start || next->end > new->end)
+ if (next->start < new->start || next->end > new->end) {
+ ret_res = next;
goto out;
+ }
if (!next->sibling)
break;
if (next->sibling->start > new->end)
break;
}
- result = 0;
+ ret_res = NULL;
new->parent = parent;
new->sibling = next->sibling;
@@ -428,11 +428,53 @@ int insert_resource(struct resource *par
}
out:
+ return ret_res;
+}
+
+/**
+ * insert_resource_shrink_to_fit - Inserts a resource in the resource tree
+ * @parent: parent of the new resource
+ * @new: new resource to insert
+ */
+void insert_resource_shrink_to_fit(struct resource *root, struct resource *new)
+{
+ write_lock(&resource_lock);
+ while (new->start && !new->parent) {
+ struct resource *conflict;
+
+ conflict = __insert_resource(root, new);
+ if (!conflict)
+ break;
+ if (conflict->start < new->start) {
+ new->start = conflict->end;
+ continue;
+ }
+ if (conflict->end > new->end) {
+ new->end = conflict->start;
+ continue;
+ }
+ }
write_unlock(&resource_lock);
- return result;
}
/**
+ * insert_resource - Inserts a resource in the resource tree
+ * @parent: parent of the new resource
+ * @new: new resource to insert
+ *
+ * Returns 0 on success, -EBUSY if the resource can't be inserted.
+ */
+int insert_resource(struct resource *parent, struct resource *new)
+{
+ struct resource *res_conflict;
+
+ write_lock(&resource_lock);
+ res_conflict = __insert_resource(parent, new);
+ write_unlock(&resource_lock);
+
+ return res_conflict ? -EBUSY : 0;
+}
+/**
* adjust_resource - modify a resource's start and size
* @res: resource to modify
* @start: new start value
next prev parent reply other threads:[~2008-08-30 3:07 UTC|newest]
Thread overview: 94+ messages / expand[flat|nested] mbox.gz Atom feed top
2008-08-28 23:26 Linux 2.6.27-rc5 Linus Torvalds
2008-08-29 15:42 ` cpu time oddity (was Re: Linux 2.6.27-rc5) Alistair John Strachan
2008-08-29 15:56 ` Alistair John Strachan
2008-08-29 17:13 ` Linux 2.6.27-rc5 Rafael J. Wysocki
2008-08-29 19:57 ` Linux 2.6.27-rc5: System boot regression caused by commit a2bd7274b47124d2fc4dfdb8c0591f545ba749dd Rafael J. Wysocki
2008-08-29 21:13 ` Yinghai Lu
2008-08-29 21:19 ` Yinghai Lu
2008-08-29 22:32 ` Rafael J. Wysocki
2008-08-29 22:31 ` Rafael J. Wysocki
2008-08-29 23:24 ` Yinghai Lu
2008-08-30 0:08 ` Linus Torvalds
2008-08-30 0:11 ` Yinghai Lu
2008-08-30 0:45 ` Linus Torvalds
2008-08-30 1:11 ` Linus Torvalds
2008-08-30 1:30 ` Yinghai Lu
2008-08-30 2:33 ` Linus Torvalds
2008-08-30 2:56 ` Linus Torvalds
2008-08-30 3:07 ` Yinghai Lu [this message]
2008-08-30 3:24 ` Linus Torvalds
2008-08-30 4:41 ` Yinghai Lu
2008-08-30 5:02 ` Yinghai Lu
2008-08-30 5:52 ` Linus Torvalds
2008-08-30 6:18 ` Linus Torvalds
2008-08-30 8:02 ` Yinghai Lu
2008-08-30 5:22 ` Yinghai Lu
2008-08-30 6:11 ` Linus Torvalds
2008-08-30 3:15 ` Linus Torvalds
2008-08-30 3:00 ` Yinghai Lu
2008-08-30 3:10 ` Linus Torvalds
2008-08-30 1:14 ` Yinghai Lu
2008-08-30 2:16 ` Linus Torvalds
2008-08-30 2:29 ` Yinghai Lu
2008-08-30 0:20 ` Yinghai Lu
2008-08-30 0:27 ` Yinghai Lu
2008-08-30 13:32 ` Rafael J. Wysocki
2008-08-30 16:05 ` Yinghai Lu
2008-08-30 17:14 ` Rafael J. Wysocki
2008-08-30 17:55 ` Yinghai Lu
2008-08-30 18:11 ` Yinghai Lu
2008-08-30 19:06 ` Yinghai Lu
2008-08-30 19:51 ` Rafael J. Wysocki
2008-08-30 20:10 ` Yinghai Lu
2008-08-29 21:44 ` Linus Torvalds
2008-08-29 22:30 ` Rafael J. Wysocki
2008-08-30 17:39 ` Linus Torvalds
2008-08-30 18:07 ` Yinghai Lu
2008-08-30 18:43 ` Linus Torvalds
2008-08-30 19:10 ` Yinghai Lu
2008-08-30 19:31 ` Linus Torvalds
2008-08-30 20:14 ` Yinghai Lu
2008-08-30 20:38 ` Yinghai Lu
2008-08-30 20:46 ` Rafael J. Wysocki
2008-08-30 21:12 ` Yinghai Lu
2008-08-30 21:13 ` Yinghai Lu
2008-08-30 21:34 ` Rafael J. Wysocki
2008-08-30 21:49 ` Yinghai Lu
2008-08-31 1:10 ` Yinghai Lu
2008-08-31 12:27 ` Rafael J. Wysocki
2008-08-31 17:42 ` Linus Torvalds
2008-08-31 17:54 ` Yinghai Lu
2008-08-31 18:03 ` Linus Torvalds
2008-08-31 21:03 ` Yinghai Lu
2008-09-01 17:53 ` Linus Torvalds
2008-08-30 22:41 ` Linus Torvalds
2008-08-30 22:50 ` Yinghai Lu
2008-08-30 23:28 ` Linus Torvalds
2008-08-30 23:39 ` Yinghai Lu
2008-08-31 0:27 ` Yinghai Lu
2008-08-31 0:50 ` Yinghai Lu
2008-08-31 3:00 ` Linus Torvalds
2008-08-31 3:53 ` Yinghai Lu
2008-08-31 3:58 ` Linus Torvalds
2008-08-31 4:12 ` Linus Torvalds
2008-08-30 19:14 ` Linus Torvalds
2008-08-30 19:26 ` Yinghai Lu
2008-08-30 19:41 ` Linus Torvalds
2008-08-30 19:48 ` Yinghai Lu
2008-08-30 19:29 ` Rafael J. Wysocki
2008-08-30 19:29 ` Yinghai Lu
2008-08-30 19:20 ` Rafael J. Wysocki
2008-08-29 22:34 ` Jeff Garzik
2008-08-29 22:47 ` Rafael J. Wysocki
2008-08-30 10:34 ` [PATCH] Re: Linux 2.6.27-rc5 Helge Deller
2008-08-30 20:21 ` Rafael J. Wysocki
2008-09-01 22:28 ` Al Viro
2008-08-31 23:27 ` J.A. Magallón
[not found] ` <328e43f00808311825r71164541pd4159a4a33b9b038@mail.gmail.com>
2008-09-01 7:12 ` Francois Romieu
2008-08-30 6:13 Linux 2.6.27-rc5: System boot regression caused by commit a2bd7274b47124d2fc4dfdb8c0591f545ba749dd David Witbrodt
2008-08-30 6:21 ` Linus Torvalds
2008-08-30 6:58 David Witbrodt
2008-08-30 23:29 David Witbrodt
2008-08-31 0:16 ` Yinghai Lu
2008-08-31 1:25 David Witbrodt
2008-08-31 2:17 ` Yinghai Lu
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=86802c440808292007t3588edfnef95b723320ff023@mail.gmail.com \
--to=yhlu.kernel@gmail.com \
--cc=akpm@linux-foundation.org \
--cc=dawitbro@sbcglobal.net \
--cc=htejun@gmail.com \
--cc=jeff@garzik.org \
--cc=kernel-testers@vger.kernel.org \
--cc=linux-kernel@vger.kernel.org \
--cc=mingo@elte.hu \
--cc=rjw@sisk.pl \
--cc=torvalds@linux-foundation.org \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
@@ -363,32 +363,30 @@ int allocate_resource(struct resource *r
* @new: new resource to insert
*
- * Returns 0 on success, -EBUSY if the resource can't be inserted.
+ * Returns NULL on success, or first conflict resource.
*
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox
all inboxes | Powered by JetHome®